# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

module Backend
  module Api
    module Schema
      @[GraphQL::Object]
      # Student model
      class Student < GraphQL::BaseObject
        include Helpers

        db_object Db::Student, Int32

        @[GraphQL::Field]
        # Student's user
        def user(context : Context) : User
          # User.new(@model.user)
          User.new(context.loaders.user.load(@model.user_id.to_i32).not_nil!)
        end

        @[GraphQL::Field]
        # Student's vote
        def vote(context : Context) : Vote?
          # @model.vote.try { |v| Vote.new(v) }
          context.loaders.student_vote.load(@model).try { |v| Vote.new(v) }
        end
      end

      # @[GraphQL::InputObject]
      # # Student base input
      # class StudentInput < GraphQL::BaseInputObject
      #   @[GraphQL::Field]
      #   def initialize
      #   end
      # end

      @[GraphQL::InputObject]
      # Student creation input
      class StudentCreateInput < GraphQL::BaseInputObject
        # Student's user ID
        getter user_id

        @[GraphQL::Field]
        def initialize(@user_id : Int32)
        end
      end
    end
  end
end
