# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

module Backend
  module Api
    module Schema
      @[GraphQL::Object]
      # Teacher model
      class Teacher < GraphQL::BaseObject
        include Helpers

        db_object Db::Teacher, Int32

        @[GraphQL::Field]
        # Teacher's user
        def user : User
          User.new(@model.user)
        end

        @[GraphQL::Field]
        # Teacher's max students
        def max_students : Int32
          @model.max_students
        end

        @[GraphQL::Field]
        # Students' votes
        def teacher_votes(context : Context) : Array(TeacherVote)
          context.loaders.teacher_vote
            .load(Db::TeacherVote.query.select(:id).where { teacher_id == @model.id }.map(&.id))
            .map { |tv| TeacherVote.new(tv.not_nil!) }
        end
      end

      @[GraphQL::InputObject]
      # Base teacher input
      class TeacherInput < GraphQL::BaseInputObject
        # Teacher's max students
        getter max_students

        @[GraphQL::Field]
        def initialize(@max_students : Int32)
        end
      end

      @[GraphQL::InputObject]
      # Teacher creation input
      class TeacherCreateInput < TeacherInput
        # Teacher's user ID
        getter user_id

        @[GraphQL::Field]
        def initialize(@user_id : Int32, max_students : Int32)
          super(max_students)
        end
      end
    end
  end
end
