# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

module Backend
  module Api
    module Schema
      @[GraphQL::Object]
      # Vote model
      class Vote < GraphQL::BaseObject
        include Helpers

        db_object Db::Vote, Int32

        @[GraphQL::Field]
        # Student who voted
        def student(context : Context) : Student
          # Student.new(@model.student)
          Student.new(context.loaders.student.load(@model.student.id).not_nil!)
        end

        @[GraphQL::Field]
        # Teacher votes for student
        def teacher_votes(context : Context) : Array(TeacherVote)
          # @model.teacher_votes.map { |tv| TeacherVote.new(tv) }
          context.loaders.teacher_vote
            .load(@model.teacher_votes.select(:id).map(&.id))
            .map { |tv| TeacherVote.new(tv.not_nil!) }
        end
      end

      @[GraphQL::InputObject]
      # Vote creation input
      class VoteCreateInput < GraphQL::BaseInputObject
        # Teacher IDs student votes for
        getter teacher_ids

        @[GraphQL::Field]
        def initialize(@teacher_ids : Array(Int32))
        end
      end
    end
  end
end
